import type {SearchOptions} from '../support/helpers';
import {createMultipleWorkGroups, createWorkGroup, findGroupRow, submitGroupSearch} from '../support/helpers';
import {randomGroupData} from '../support/rand';

const school = Cypress.env('SCHOOL');

describe('Work group handling tests', () => {
  it('creates and deletes a work group correctly', () => {
    cy.login();
    const workGroup = createWorkGroup(school);
    const searchOptions: SearchOptions = {
      schoolSelection: school,
      quickSearch: workGroup.name,
    };
    submitGroupSearch(searchOptions);
    const tr = findGroupRow(workGroup, {assertExists: true});
    tr.find('td:first-child input').click();
    cy.get('.uGrid__header .uButton')
      .eq(2)
      .should('not.have.attr', 'disabled');
    cy.get('.uGrid__header .uButton')
      .eq(2)
      .click();
    cy.get('.uConfirmDialog .uButton--primary')
      .click();
    cy.get('.uGrid__tableWrapper tbody tr')
      .should('have.length', 0);
    submitGroupSearch(searchOptions);
    cy.get('.uGrid__tableWrapper tbody tr')
      .should('have.length', 0);
  });

  it('creates and edits work group correctly', () => {
    cy.login();
    const originalWorkGroupData = randomGroupData();
    const editedWorkGroupData = randomGroupData();
    createWorkGroup(school, originalWorkGroupData);
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData, {assertExists: true})
      .find('td:first-child input')
      .click();
    cy.get('.uGrid__header .uButton')
      .eq(1)
      .within($button => {
        cy.wrap($button).should('not.have.attr', 'disabled');
        cy.wrap($button).click();
      });
    cy.url().should('contain', `${Cypress.config().baseUrl}/workgroup/edit/${school}-${originalWorkGroupData.name}`);
    cy.get('input[name="description"]')
      .clear({scrollBehavior: 'center'})
      .type(editedWorkGroupData.name, {scrollBehavior: 'center'});
    cy.get('input[name="name"]')
      .clear({scrollBehavior: 'center'})
      .type(editedWorkGroupData.name, {scrollBehavior: 'center'});
    const url = new URL(Cypress.config().baseUrl as string);
    cy.intercept({
      method: 'PATCH',
      hostname: url.hostname,
    }).as('patch');
    cy.get('.listView__header__buttons [type="button"]')
      .eq(0)
      .click();
    cy.wait('@patch');
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData).then($row => {
      cy.wrap($row).should('have.length', 0);
    });
    findGroupRow(editedWorkGroupData, {assertExists: true}).then($row => {
      if (editedWorkGroupData.name) {
        cy.wrap($row.find('td:nth-child(4)')).should('contain.text', editedWorkGroupData.name);
      }
    });
  });

  it('creates workgroup and edits workgroups lms state and search for it correctly', () => {
    cy.login();
    const originalWorkGroupData = randomGroupData();
    createWorkGroup(school, originalWorkGroupData);
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData, {assertExists: true})
      .find('td:first-child input')
      .click();
    cy.get('.uGrid__header .uButton')
      .eq(1)
      .within($button => {
        cy.wrap($button).should('not.have.attr', 'disabled');
        cy.wrap($button).click();
      });
    cy.url().should('contain', `${Cypress.config().baseUrl}/workgroup/edit/`);
    cy.get('input[name="description"]')
      .clear({scrollBehavior: 'center'})
      .type(originalWorkGroupData.name, {scrollBehavior: 'center'});
    cy.get('input[name="isLmsGroup"]')
      .check();
    const url = new URL(Cypress.config().baseUrl as string);
    cy.intercept({
      method: 'PATCH',
      hostname: url.hostname,
    }).as('patch');
    cy.get('.listView__header__buttons [type="button"]')
      .eq(0)
      .click();
    cy.wait('@patch');
    searchOptions.isLmsGroup = true;
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData).then($row => {
      cy.wrap($row).should('have.length', 1);
    });
    findGroupRow(originalWorkGroupData, {assertExists: true}).then($row => {
      if (originalWorkGroupData.name) {
        cy.wrap($row.find('td:nth-child(4)')).should('contain.text', originalWorkGroupData.name);
      }
    });
  });

  it('creates workgroup and edits workgroups Iserv state and search for it correctly', () => {
    cy.login();
    const originalWorkGroupData = randomGroupData();
    createWorkGroup(school, originalWorkGroupData);
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData, {assertExists: true})
      .find('td:first-child input')
      .click();
    cy.get('.uGrid__header .uButton')
      .eq(1)
      .within($button => {
        cy.wrap($button).should('not.have.attr', 'disabled');
        cy.wrap($button).click();
      });
    cy.url().should('contain', `${Cypress.config().baseUrl}/workgroup/edit/`);
    cy.get('input[name="description"]')
      .clear({scrollBehavior: 'center'})
      .type(originalWorkGroupData.name, {scrollBehavior: 'center'});
    cy.get('input[name="isIservGroup"]')
      .check();
    const url = new URL(Cypress.config().baseUrl as string);
    cy.intercept({
      method: 'PATCH',
      hostname: url.hostname,
    }).as('patch');
    cy.get('.listView__header__buttons [type="button"]')
      .eq(0)
      .click();
    cy.wait('@patch');
    searchOptions.isIservGroup = true;
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData).then($row => {
      cy.wrap($row).should('have.length', 1);
    });
    findGroupRow(originalWorkGroupData, {assertExists: true}).then($row => {
      if (originalWorkGroupData.name) {
        cy.wrap($row.find('td:nth-child(4)')).should('contain.text', originalWorkGroupData.name);
      }
    });
  });

  it('creates and deletes multiple work groups correctly', () => {
    cy.login();
    const workGroups = createMultipleWorkGroups(school, 2);
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(workGroups[0], {assertExists: true}).then($row => {
      cy.wrap($row).find('td:first-child input').click();
    });
    findGroupRow(workGroups[1], {assertExists: true}).then($row => {
      cy.wrap($row).find('td:first-child input').click();
    });
    cy.get('.uGrid__header .uButton')
      .eq(2)
      .should('not.have.attr', 'disabled');
    cy.get('.uGrid__header .uButton')
      .eq(2)
      .click();
    cy.get('.uConfirmDialog .uButton--primary')
      .click();
  });
});
