import type {
  GroupListResponseModel,
  ListViewConfig,
  SchoolGroupType,
  ToggleActiveAction,
  ToggleActiveProp,
} from '@/helpers/models';
import {fetchAuthenticated} from '@/helpers/retrieveData';
import {useErrorStore} from '@/stores/error';
import i18next from 'i18next';
import type {UGridRow} from '@univention/univention-veb';
import {fetchMockData, getGroupListResponseModels, listViewConfig, members} from '@/helpers/mocks';

export const needMock = import.meta.env.VITE_USE_REAL_BACKEND !== 'true';

export const getError = async(response: Response): Promise<{detail: unknown} | null> => {
  if (!response.ok) {
    const text = await response.text() || response.statusText;
    let detail = {
      detail: text,
    };
    try {
      const json = JSON.parse(text);
      if (json && typeof json === 'object' && 'detail' in json) {
        detail = json;
      }
    } catch (err) {
      // Either not a JSON response, or unparsable. We'll just ignore this.
    }
    return detail;
  }
  return null;
};

export type SaveError = {
  type: 'generic';
  message: string;
} | {
  type: 'fieldErrors';
  errors: {
    field: string;
    message: string;
  }[];
};
export const getSaveError = async(response: Response, context: 'updateWorkgroup' | 'createWorkgroup' | 'updateSchoolClass'): Promise<SaveError | null> => {
  const errorJson = await getError(response);
  if (!errorJson) {
    return null;
  }
  if (response.status === 422) {
    // FIXME validate against json.schema
    if (Array.isArray(errorJson.detail)) {
      const detail: {loc: string[]; msg: string; type: string}[] = errorJson.detail;
      const error: SaveError = {
        type: 'fieldErrors',
        errors: [],
      };
      for (const iDetail of detail) {
        const field = iDetail.loc[iDetail.loc.length - 1];
        if (field) {
          error.errors.push({
            field,
            message: iDetail.msg,
          });
        }
      }
      return error;
    }
  }
  if (response.status === 404) {
    return {
      type: 'generic',
      message: i18next.t(`dataAccess.${context}.error.404`),
    };
  }
  return {
    type: 'generic',
    message: String(errorJson.detail),
  };
};

export const fetchListViewConfig = async(): Promise<ListViewConfig> => {
  if (needMock) {
    return fetchMockData(listViewConfig, 'fetchListViewConfig');
  }

  const answer = await fetchAuthenticated('/ucsschool/bff-groups/v1/pageconf/listView');
  const error = await getError(answer);
  if (error) {
    const errorStore = useErrorStore();
    const title = i18next.t('dataAccess.fetchListViewConfig.error.title');
    errorStore.push({
      title,
      message: JSON.stringify(error.detail),
      unRecoverable: true,
    });
    return Promise.reject(answer);
  }
  return await answer.json();
};

export const fetchGroups = async(
  school: string,
  idiSchoolyear: string,
  divisClasstype: string,
  groupSourceId: string,
  disabled: string,
  isLmsGroup: string,
  iServEnabled: string,
  quickSearch: string,
  limit?: number,
): Promise<GroupListResponseModel[] | null> => {
  if (needMock) {
    return fetchMockData(getGroupListResponseModels(), 'fetchGroups');
  }

  const urlSearchParams = new URLSearchParams({school});
  if (idiSchoolyear !== '') {
    urlSearchParams.append('idiSchoolyear', idiSchoolyear);
  }
  if (divisClasstype !== '') {
    urlSearchParams.append('divisClasstype', divisClasstype);
  }
  if (groupSourceId !== '') {
    urlSearchParams.append('groupSourceId', groupSourceId);
  }
  if (disabled !== '') {
    urlSearchParams.append('disabled', (disabled === 'disabled').toString());
  }
  if (isLmsGroup !== '') {
    urlSearchParams.append('isLmsGroup', (isLmsGroup === 'enabled').toString());
  }
  if (iServEnabled !== '') {
    urlSearchParams.append('isIservGroup', (iServEnabled === 'enabled').toString());
  }
  if (quickSearch !== '') {
    urlSearchParams.append('quickSearch', quickSearch);
  }
  if (limit) {
    urlSearchParams.append('limit', `${limit}`);
  }

  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/groups?${urlSearchParams.toString()}`);
  const error = await getError(answer);
  if (error) {
    if (typeof error.detail === 'object' && error.detail !== null && 'type' in error.detail) {
      const detail = error.detail as {type: string};
      if (detail.type === 'SearchLimitReached') {
        return null;
      }
    }
    const errorStore = useErrorStore();
    const title = i18next.t('dataAccess.fetchGroups.error.title');
    errorStore.push({
      title,
      message: JSON.stringify(error.detail),
    });
    return Promise.reject(answer);
  }
  return await answer.json();
};


export const deleteGroups = async(groups: GroupListResponseModel[]): Promise<{id: string; error: string}[]> => {
  if (needMock) {
    return fetchMockData([
      {
        id: groups[0]?.id as string,
        error: 'some error',
      },
    ], 'deleteGroups');
  }

  const responses = await Promise.all(groups.map(group => fetchAuthenticated(`/ucsschool/bff-groups/v1/${group.schoolGroupType}/${group.id}`, {
    method: 'DELETE',
  })));
  const errors = await Promise.all(responses.map(x => getError(x)));
  const fails: {id: string; error: string}[] = [];
  for (let x = 0; x < responses.length; x++) {
    const error = errors[x];
    if (error) {
      fails.push({
        id: groups[x]?.id as string,
        error: JSON.stringify(error.detail),
      });
    }
  }
  return fails;
};

export const fetchMembers = async(school: string, values: Record<string, unknown>): Promise<UGridRow[]> => {
  if (needMock) {
    return fetchMockData(members, 'fetchMembers');
  }

  const urlSearchParams = new URLSearchParams({school});
  for (const key in values) {
    const value = values[key];
    if (typeof value === 'string' && value !== '') {
      urlSearchParams.append(key, value);
    }
  }
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/users?${urlSearchParams.toString()}`);
  const error = await getError(answer);
  if (error) {
    const errorStore = useErrorStore();
    const title = i18next.t('dataAccess.fetchMembers.error.title');
    errorStore.push({
      title,
      message: JSON.stringify(error.detail),
    });
    return Promise.reject(answer);
  }
  return await answer.json();
};

export const toggleGroupPropActivationStates = async(groupsToToggle: {id: string; schoolGroupType: SchoolGroupType}[], action: ToggleActiveAction, prop: ToggleActiveProp): Promise<{id: string; error: string}[]> => {
  if (needMock) {
    return fetchMockData([
      {
        id: groupsToToggle[0]?.id as string,
        error: 'some error',
      },
    ], 'toggleGroupPropActivationStates');
  }

  const responses = await Promise.all(groupsToToggle.map(group => fetchAuthenticated(`/ucsschool/bff-groups/v1/${prop}/${group.schoolGroupType}/${group.id}/${action}`, {
    method: 'PATCH',
  })));
  const errors = await Promise.all(responses.map(x => getError(x)));
  const fails: {id: string; error: string}[] = [];
  for (let x = 0; x < responses.length; x++) {
    const error = errors[x];
    if (error) {
      fails.push({
        id: groupsToToggle[x]?.id as string,
        error: JSON.stringify(error.detail),
      });
    }
  }
  return fails;
};
